function test_cbs

% This file is part of CBS.
%
% CBS is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% CBS is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with CBS.  If not, see <http://www.gnu.org/licenses/>.

close all;

%compare_with_spline_toolbox_onedim
%compare_with_spline_toolbox_twodim
%test_coloc
%test_bending
test_coloc_deriv

function compare_with_spline_toolbox_onedim %#ok<*DEFNU>

ctrlpts = rand(1,7);
sp = spmak(-7:2:13, ctrlpts);
cbs = cbs_create(-1, 7, 7, 1);

t = linspace(-1,7,1000);

% Evaluation
x1 = fnval(sp, t);
x2 = cbs_eval(cbs, ctrlpts, t);

figure(1);
plot(t, x1);
hold on;
plot(t, x2, 'r:');
title('Evaluation');

fprintf('[Evaluation] Difference norm: %f\n', norm(x1-x2));

% First derivative
sp_dt = fnder(sp, 1);

x1_dt = fnval(sp_dt, t);
x2_dt = cbs_eval(cbs, ctrlpts, t, 1);

figure(2);
plot(t, x1_dt);
hold on;
plot(t, x2_dt, 'r:');
title('First derivative');

fprintf('[First derivative] Difference norm: %f\n', norm(x1_dt-x2_dt));

% Second derivative
sp_dtt = fnder(sp, 2);

x1_dtt = fnval(sp_dtt, t);
x2_dtt = cbs_eval(cbs, ctrlpts, t, 2);

figure(3);
plot(t, x1_dtt);
hold on;
plot(t, x2_dtt, 'r:');
title('Second derivative');

fprintf('[Second derivative] Difference norm: %f\n', norm(x1_dtt-x2_dtt));

function compare_with_spline_toolbox_twodim

ctrlpts = rand(2,7);
sp = spmak(-7:2:13, ctrlpts);

cbs = cbs_create(-1, 7, 7, 2);

t = linspace(-1,7,1000);

% Evaluation
tmp = fnval(sp, t);
x1 = tmp(1,:);
y1 = tmp(2,:);
[x2 y2] = cbs_eval(cbs, ctrlpts, t);

figure(1);
plot(x1, y1);
hold on;
plot(x2, y2, 'r:');
title('Evaluation');

fprintf('[Evaluation] Difference norm: %f\n', sum(sqrt((x2-x1).^2+(y2-y1).^2)));

% First derivative
sp_dt = fnder(sp, 1);

tmp = fnval(sp_dt, t);
x1_dt = tmp(1,:);
y1_dt = tmp(2,:);

[x2_dt y2_dt] = cbs_eval(cbs, ctrlpts, t, 1);

figure(2);
plot(x1_dt, y1_dt);
hold on;
plot(x2_dt, y2_dt, 'r:');
title('First derivative');

fprintf('[First derivative] Difference norm: %f\n', sum(sqrt((x2_dt-x1_dt).^2+(y2_dt-y1_dt).^2)));

% Second derivative
sp_dtt = fnder(sp, 2);

tmp = fnval(sp_dtt, t);
x1_dtt = tmp(1,:);
y1_dtt = tmp(2,:);
[x2_dtt y2_dtt] = cbs_eval(cbs, ctrlpts, t, 2);

figure(3);
plot(x1_dtt, y1_dtt);
hold on;
plot(x2_dtt, y2_dtt, 'r:');
title('Second derivative');

fprintf('[Second derivative] Difference norm: %f\n', sum(sqrt((x2_dtt-x1_dtt).^2+(y2_dtt-y1_dtt).^2)));

function test_coloc

cbs = cbs_create(-1, 10, 81, 1);
ctrlpts = 5*(rand(1,81)-0.5);

t = linspace(-1, 10);

colmat = cbs_coloc(cbs, t);

y1 = cbs_eval(cbs, ctrlpts, t);

y2 = (colmat * ctrlpts')';

fprintf('Difference direct eval./eval with coloc: %f\n', norm(y1-y2));

figure(1);
spy(colmat);



function test_bending
tmin = -1.23;
tmax = 45.6;
npts = 78;

cbs = cbs_create(tmin, tmax, npts, 1);
cp = rand(1, npts);


bendmat = cbs_bending(cbs, ones(1,npts-3));

gt = quadgk(@(t)fun_bend(cbs, cp, t), tmin, tmax, 'MaxIntervalCount', 1000000, 'RelTol', 1.0e-8);

c = cp * bendmat * cp';

fprintf('Ground truth: %f - Bendmat: %f - Diff: %f\n', gt, c, gt-c);

function val = fun_bend(cbs, cp, t)
val = cbs_eval(cbs, cp, t, 2).^2;



function test_coloc_deriv
tmin = -1.23;
tmax = 45.6;
npts = 78;

cbs = cbs_create(tmin, tmax, npts, 1);
cp = rand(1, npts);

t = rand(1,100)*(tmax-tmin)+tmin;

col0 = cbs_coloc_deriv(cbs, t, 0);
col1 = cbs_coloc_deriv(cbs, t, 1);
col2 = cbs_coloc_deriv(cbs, t, 2);

gt0 = cbs_eval(cbs, cp, t);
gt1 = cbs_eval(cbs, cp, t, 1);
gt2 = cbs_eval(cbs, cp, t, 2);

c0 = (col0*(cp'))';
c1 = (col1*(cp'))';
c2 = (col2*(cp'))';

fprintf('dt0: %f - dt1: %f - dt2: %f\n', norm(gt0-c0), norm(gt1-c1), norm(gt2-c2));
